from django.db.models.signals import post_save
from django.dispatch import receiver

from base64 import urlsafe_b64encode as ub64
from hashlib import sha1
from time import time

from courses.models import Course, CourseStudent
from courses.certification.models import CourseCertification, CertificateTemplate


@receiver(post_save, sender=CourseStudent)
def course_student_created_or_updated(sender, created, instance: CourseStudent, **kwargs):

    if created:
        link_hash = ub64(sha1((str(time()) + instance.user.last_name).encode('utf-8')).digest()[0:6])
        receipt = CourseCertification(course_student=instance,
                                      type=CourseCertification.TYPES[0][0],
                                      is_valid=True, link_hash=link_hash.decode())
        receipt.save()


@receiver(post_save, sender=Course)
def course_created(sender, created, instance, **kwargs):

    if created:
        for type_ in CertificateTemplate.TYPES:
            ct = CertificateTemplate(course=instance, document_type=type_[0])
            ct.save()
