from rest_framework import serializers

from baquara.users.serializers import SimpleUserSerializer

from courses.serializers import BasicCourseSerializer

from discussion.serializers import ForumSumarySerializer

from .models import Classroom, Event


class EventSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = Event
        fields = '__all__'


class BasicClassroomSerializer(serializers.ModelSerializer):

    assistants = SimpleUserSerializer(read_only=True, many=True)
    coordinators = SimpleUserSerializer(read_only=True, many=True)

    class Meta:
        model = Classroom
        fields = (
            'id',
            'name',
            'description',
            'image',
            'assistants',
            'coordinators',
        )


class ClassroomSerializer(serializers.ModelSerializer):

    assistants = SimpleUserSerializer(read_only=True, many=True)
    coordinators = SimpleUserSerializer(read_only=True, many=True)
    events = EventSerializer(read_only=True, many=True)
    courses = BasicCourseSerializer(read_only=True, many=True)
    forum = ForumSumarySerializer(read_only=True)
    can_edit = serializers.SerializerMethodField()

    class Meta:
        model = Classroom
        fields = (
            'name',
            'description',
            'image',
            'assistants',
            'coordinators',
            'courses',
            'access_groups',
            'group',
            'events',
            'chat_url',
            'forum',
            'can_edit',
        )
        read_only_fields = ['image']
        depth = 1

    def get_can_edit(self, obj):
        request = self.context.get("request", None)
        if request and hasattr(request, "user"):
            return obj.is_assistant_or_coordinator(request.user) \
                or request.user.is_superuser
