from rest_framework import viewsets, mixins
from rest_framework.permissions import IsAuthenticated
from courses.models import Course
from courses.course_material.models import CourseMaterial, File as CourseMaterialFile
from courses.course_material.serializers import CourseMaterialSerializer, FilesSerializer


class CourseMaterialViewSet(viewsets.ModelViewSet):
    model = CourseMaterial
    queryset = CourseMaterial.objects.all()
    serializer_class = CourseMaterialSerializer
    permission_classes = (IsAuthenticated,)
    lookup_field = 'course'
    filter_fields = ('course__id',)

    def pre_save(self, obj):
        # Get Question vote usign kwarg as questionId
        if 'course' in self.kwargs:
            obj.course = Course.objects.get(id=int(self.kwargs['course']))
            self.kwargs['course'] = obj.course
        return super(CourseMaterialViewSet, self).pre_save(obj)


class CourseMaterialFileViewSet(mixins.DestroyModelMixin, viewsets.GenericViewSet):
    model = CourseMaterialFile
    queryset = CourseMaterialFile.objects.all()
    serializer_class = FilesSerializer
    permission_classes = (IsAuthenticated,)
