from rest_framework import permissions
from .models import Course, CourseProfessor, CourseAuthor


class IsProfessorCoordinatorOrAdminPermission(permissions.BasePermission):
    """
    Custom permission to only allow course coordinator edit and view Course and CourseProfessors objects instances
    """
    def has_object_permission(self, request, view, obj):
        # TODO Verify if all this cases make sense
        if request.user and request.user.is_superuser:
            return True
        elif request.user.is_authenticated and isinstance(obj, CourseProfessor) and obj.course.get_professor_role(request.user) == 'coordinator':
            return True
        elif request.user.is_authenticated and isinstance(obj, Course) and obj.get_professor_role(request.user) == 'coordinator':
            return True
        elif request.user.is_authenticated and isinstance(obj, CourseAuthor) and obj.course.get_professor_role(request.user) == 'coordinator':
            return True


class IsProfessorCoordinatorOrAdminPermissionOrReadOnly(IsProfessorCoordinatorOrAdminPermission):
    """
    Custom permission to only allow course coordinator edit Course and CourseProfessors objects instances
    """
    def has_object_permission(self, request, view, obj):
        # Read permissions are allowed to any request,
        # so we'll always allow GET, HEAD or OPTIONS requests.
        if request.method in permissions.SAFE_METHODS:
            return True
        return super().has_object_permission(request, view, obj)


class IsAdmin(permissions.BasePermission):
    """
    Custom permission to only allow staff or superusers access.
    """
    def has_permission(self, request, view):
        if request.user and request.user.is_staff:
            return True
        elif request.user and request.user.is_superuser:
            return True


class IsAssistantOrCoordinatorOrAdminOrRecipient(permissions.BasePermission):
    """
    Check if a user can send or read messages.
    """
    def has_object_permission(self, request, view, obj):
        if request.user and request.user.is_superuser:
            return True
        elif (request.method in permissions.SAFE_METHODS and 
                (request.user in obj.users.all() or 
                request.user.groups.all().intersection(obj.groups.all()).exists())):
            return True
        elif obj.course:
            return obj.course.is_assistant_or_coordinator(request.user)
        elif obj.classrooms:
            # The user must be assistant or coordinator of all classroom he want to send the message
            for classroom in obj.classrooms.all():
                if not classroom.is_assistant_or_coordinator(request.user):
                    return False


class IsAdminOrReadOnly(IsAdmin):
    """
    Custom permission to only allow not safe methods to admin.
    """
    def has_permission(self, request, view):
        # Read permissions are allowed to any request,
        # so we'll always allow GET, HEAD or OPTIONS requests.
        if request.method in permissions.SAFE_METHODS:
            return True
        return super(IsAdminOrReadOnly, self).has_permission(request, view)
