from rest_framework import serializers
from actstream.models import Action
from courses.reports.serializers import UserStatsSerializer
from courses.serializers import SimpleUserSerializer

from django.contrib.auth.models import User
from django.contrib.auth import get_user_model
User = get_user_model()

from courses.classroom.models import Classroom
from courses.classroom.serializers import BasicClassroomSerializer

from discussion.models import Comment, Topic

from datetime import datetime

class UserActionsSerializer(serializers.Serializer):
    verb = serializers.CharField(required=True, max_length=100)
    action_object_id = serializers.IntegerField(required=False)
    action_object_type = serializers.CharField(required=False, max_length=100)
    target_id = serializers.IntegerField(required=False)
    target_type = serializers.CharField(required=False, max_length=100)


class UserAccessSerializer(serializers.Serializer):
    area = serializers.CharField(required=True, max_length=100)


class GenericRelatedField(serializers.Field):
    def to_representation(self, value):
        if isinstance(value, User):
            return SimpleUserSerializer(value).data
        if isinstance(value, Classroom):
            return BasicClassroomSerializer(value).data
        return str(value)


class ActionSerializer(serializers.ModelSerializer):
    actor = GenericRelatedField(read_only=True)
    target = GenericRelatedField(read_only=True)
    action_object = GenericRelatedField(read_only=True)
    verb = GenericRelatedField(read_only=True)
    action_phrase = serializers.SerializerMethodField()
    timestamp = serializers.SerializerMethodField()

    class Meta:
        model = Action
        fields = ('actor', 'target', 'action_object', 'verb', 'action_phrase', 'timestamp')

    def get_timestamp(self, obj):
        timestamp_str = obj.timestamp.strftime('%d-%m-%Y %H:%M:%S')
        date, hour = timestamp_str.split()
        return date + ' às ' + hour

    def get_object_type(self, obj):
        if isinstance(obj, Classroom):
            return ' na sala '
        if isinstance(obj, Comment):
            return ' no comentário '
        if isinstance(obj, Topic):
            return ' no tópico '
        return ''

    def get_action_phrase(self, obj):
        verbs = {
            'access': 'acessou',
            'answered': 'respondeu',
            'created': 'criou',
            'reacted': 'reagiu'
        }

        action_phrase = ''

        action_phrase += obj.actor.name
        action_phrase += (' ' + verbs.get(obj.verb))

        if obj.target:
            action_phrase += (self.get_object_type(obj.target) + str(obj.target))
        if obj.action_object:
            action_phrase += (self.get_object_type(obj.action_object) + str(obj.action_object))

        return action_phrase
