from django.db import models
from django.contrib.auth.models import Group
from django.utils.translation import ugettext_lazy as _
from django.contrib.postgres.fields import ArrayField

from autoslug import AutoSlugField


class Workspace(models.Model):
    name = models.CharField(_('Name'), max_length=255)
    description = models.TextField(blank=True,
                                   verbose_name = _("description"),
                                   help_text = _("Enter the workspace description (optional)" ),)
    slug = AutoSlugField(_('Slug'), populate_from='name', max_length=128, editable=False, unique=True)
    groups = models.ManyToManyField(
        Group,
        verbose_name=_('groups'),
        blank=True,
        help_text=_('Groups created to enforce this workspace restrictions in several other models'),
        related_name='contract',
    )
    classes = models.ManyToManyField('Class', related_name='contract')
    unities = ArrayField(
        models.CharField(max_length=255, blank=True, null=True),
        help_text=_('Possibly cities. This field stores the user declared unities on workspace creation. This are used to distinguish some special grupos associated to the contract'),
        blank=True,
        null=True,
    )

    @property
    def unities_normalized(self):
        return filter(lambda x: x !=  '', [x.strip().rstrip().lower() for x in self.unities])

    class Meta:
        verbose_name = _('Workspace')
        verbose_name_plural = _('Workspaces')

    def __str__(self):
        return self.name
