from rest_framework import serializers

from django.contrib.auth.models import Group

from baquara.users.serializers import SimpleUserSerializer
from courses.models import Course, Class
# from courses.serializers import ClassSerializer as CoreClassSerializer
# from courses_legacy.core.serializers import ClassSerializer as CoreClassSerializer

from courses.workspaces.models import Workspace


class CourseSerializer(serializers.ModelSerializer):

    class Meta:
        model = Course
        fields = ('id', 'name')


class GroupSerializer(serializers.ModelSerializer):
    class Meta:
        model = Group
        fields = ('id', 'name')


class GroupAdminSerializer(serializers.ModelSerializer):

    users = SimpleUserSerializer(many=True, read_only=True, source="user_set")

    class Meta:
        model = Group
        fields = ('id', 'name', 'users', )


class ClassSerializer(serializers.ModelSerializer):
    course = CourseSerializer(read_only=True)

    class Meta:
        model = Class
        fields = ('id', 'name', 'course')


class WorkspaceSerializer(serializers.ModelSerializer):
    groups = GroupSerializer(many=True, read_only=True)
    classes = ClassSerializer(many=True, read_only=True)

    class Meta:
        model = Workspace
        fields = '__all__'
        depth = 1


class SimpleWorkspaceSerializer(serializers.ModelSerializer):

    class Meta:
        model = Workspace
        fields = ('id', 'name', 'description')


class WorkspaceBaseSerializerMixin(serializers.ModelSerializer):
    contract = serializers.SerializerMethodField(read_only=False, )

    def get_contract(self, obj):
        contract = obj.contract.first()
        if contract:
            return SimpleWorkspaceSerializer(obj.contract.first(),).data
        else:
            return None


class WorkspaceGroupSerializer(WorkspaceBaseSerializerMixin, GroupSerializer):

    class Meta(GroupSerializer.Meta):
        fields = ('id', 'name', 'contract')


class WorkspaceGroupAdminSerializer(WorkspaceBaseSerializerMixin, GroupAdminSerializer):

    class Meta(GroupAdminSerializer.Meta):
        fields = ('id', 'name', 'users', 'contract')


class CourseGroupSerializer(serializers.ModelSerializer):
    groups = WorkspaceGroupSerializer(many=True, read_only=True)

    class Meta:
        model = Course
        fields = ('slug', 'name', 'groups')
